@@ -1,7 +1,9 @@
-\dontrun{
+\donttest{
+if (require("INLA", quietly = TRUE)) {
+  
 # Load Gorilla data
 
-data("gorillas")
+data("gorillas", package = "inlabru")
 
 # Use RColorBrewer 
 
@@ -19,8 +21,6 @@ cmp <- coordinates ~ vegetation(map = gorillas$gcov$vegetation, model = "factor"
   spde(map = coordinates, model = pcmatern, mesh = gorillas$mesh) -
   Intercept
 
-init.tutorial() # Faster inferece for example
-
 fit <- lgcp(cmp, gorillas$nests, samplers = gorillas$boundary)
 
 # Predict SPDE and vegetation at the mesh vertex locations
@@ -85,4 +85,4 @@ vm.int <- devel.cvmeasure(joint, field, veg,
 vm.int
 
 }
-
+}
