@@ -1,14 +1,9 @@
 \donttest{
-# Load INLA
-library(INLA)
+if (require("INLA", quietly = TRUE)) {
 
 # Load the Gorilla data
 
-data(gorillas)
-
-# Use tutorial setting and thus empirical Bayes for faster inference
-
-init.tutorial()
+data(gorillas, package ="inlabru")
 
 # Plot the Gorilla nests, the mesh and the survey boundary
 
@@ -91,5 +86,6 @@ mySmooth = predict(fit, pxl, ~ mySmooth)
 
 head(mySmooth)
 ggplot() + gg(mySmooth)
-}
 
+}
+}
